"""
This files writes a BMP file with a Sierpinski triangle.
The values for the triangle are calculated from a Pascal triangle.
This is a terribly inefficient way of doing this.
Jeremie Gillet - Skill Pill: Programming with Python - Feb. 2016
"""

# returns a Pascal triangle mod 2
def pascal_mod2(size):
    pascal=[[1]]
    for i in range(size-1):
     tmp=[1]
     for j in range(i):
         tmp+=[(pascal[i][j]+pascal[i][j+1])%2]
     tmp+=[1]
     pascal+=[tmp]
    return pascal

from struct import *

# size of the image
size=256
# Calculating the Pascal triangle values (this is slow)
p=pascal_mod2(size)

# Parameters for the BMP images
bpp=32 # bit per pixel
width=size*2 # width in pixels
height=-size*2 # height in pixels
row_size=4*((bpp*width+31)//32) # number of bytes in a row
pix_array_size=row_size*abs(height) # number of bytes for all pixels

# writing the BMP file
f = open("pascal.bmp","w")

# Header (14 bytes)
f.write("BM") # Bitmap header
f.write(pack('<i',pix_array_size+14+40)) # Size of file
f.write(pack('<i',0)) # reserved
f.write(pack('<i',14+40)) # offset for beginning of data

# DIB header (40 bytes)
f.write(pack('<i',40)) # size of this header
f.write(pack('<i',width)) # width of image
f.write(pack('<i',height)) # height of image
f.write(pack('<H',1)) # number of color planes (must be 1)
f.write(pack('<H',bpp)) # number of bits per pixel
f.write(pack('<i',0)) #compression technique
f.write(pack('<i',0)) # image size
f.write(pack('<i',0)) # horizontal resolution
f.write(pack('<i',0)) # vertical resolution
f.write(pack('<i',0)) # color palette
f.write(pack('<i',0)) # important colors

# data
for row in range(size):
    for double_it in range(2): # each data point is written on 4 pixels so each row is doubled

        f.write((size-row-1)*pack('BBBB',255,255,255,255)) # padding left with white pixels

        for col in range(row+1):
            if p[row][col]==0:
                f.write(2*pack('BBBB',255,255,255,255)) # white pixel
            else:
                f.write(2*pack('BBBB',255,0,0,255)) # blue pixel

        f.write((size-row-1)*pack('BBBB',255,255,255,255)) # padding right with white pixels

f.close()